/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.quarkus.component.saga.it;

import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import org.apache.camel.CamelContext;
import org.jboss.logging.Logger;

@Path("/saga")
@ApplicationScoped
public class SagaResource {

    private static final Logger LOG = Logger.getLogger(SagaResource.class);

    private static final String COMPONENT_SAGA = "saga";

    @Inject
    CamelContext context;

    @Inject
    OrderManagerService orderManagerService;

    @Inject
    CreditService creditService;

    @Path("/load/component/saga")
    @GET
    @Produces(MediaType.TEXT_PLAIN)
    public Response loadComponentSaga() throws Exception {
        /* This is an autogenerated test */
        if (context.getComponent(COMPONENT_SAGA) != null) {
            return Response.ok().build();
        }
        LOG.warnf("Could not load [%s] from the Camel context", COMPONENT_SAGA);
        return Response.status(500, COMPONENT_SAGA + " could not be loaded from the Camel context").build();
    }

    @Path("/test")
    @GET
    @Produces(MediaType.TEXT_PLAIN)
    public Response buy() {
        try {
            // total credit is 100
            buy(20, false, false);
            buy(70, false, false);
            buy(20, false, true); // fail
            buy(5, false, false);
            buy(1, true, true); // fail

            // wait for the orders being cancelled
            Thread.sleep(500);

            if (orderManagerService.getOrders().size() == 3 && creditService.getCredit() == 5) {
                return Response.ok().build();
            } else {
                return Response.status(500, "not equal").build();
            }
        } catch (Exception e) {
            return Response.status(500, COMPONENT_SAGA + " is failed with " + e).build();
        }
    }

    private void buy(int amount, boolean failAtTheEnd, boolean shouldFail) throws RuntimeException {
        try {
            context.createFluentProducerTemplate().to("direct:saga").withHeader("amount", amount)
                    .withHeader("fail", failAtTheEnd).request();

            if (shouldFail) {
                throw new RuntimeException("Exception not thrown");
            }
        } catch (Exception ex) {
            if (!shouldFail) {
                throw new RuntimeException("Unexpected exception");
            }
        }
    }
}
